;**********************************************************************
;                                                                     *
;    Filename:	    speedo.asm                                        *
;    Date:        	December 5, 2010                                  *
;    File Version:  1.00                                              *
;                                                                     *
;    Author:        Brian Kelly                                       *
;    Company:                                                         *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;    Converts pulse rate representing KPH to MPH.                     *
;    Conversion factor is 1 MPH = 1.60934 KPH                         *
;                         1 KPH = 0.62137 MPH                         *
;    Two modes: with GP1 = 0, converts to MPH, 1 = keep in KPH        *
;    Two preciions: with GP3 = 0 uses 3/8 division (~2% error)        *
;                   with GP3 = 1 uses 64/103 disvision (~0.05% error) *
;                                                                     *
;**********************************************************************

	list		p=10F202			;list directive to define processor
	#include	<p10F202.inc>		;processor specific variable definitions

	__CONFIG	_MCLRE_OFF & _CP_ON & _WDT_OFF

;aliases for GPIO bit numbers
#define	Output 0
#define Mode 1
#define TachoIn 2
#define Precision 3

;Variables
	cblock	0x10
OldPinState							;used to detect rising edge on tacho input
PulseCount
CycleCount
	endc

;**********************************************************************
	ORG		0x000					;code begins here with osc cal in W
	
initialise
	movwf   OSCCAL					;update register with factory cal value 
	movlw	0x0E
	tris	GPIO					;GPIO.0 is o/p, other pins are inputs
	clrf	OldPinState				;prepare for use
	clrf	PulseCount
	clrf	CycleCount

;if Mode = 1 just copy tacho input to the output (no conversion)
loop	
	btfss	GPIO,Mode				;skip if converting to MPH
	goto	convert
readTacho
	btfss	GPIO,TachoIn			;copy the Tacho input directly to Output
	goto	OutLo
OutHi
	bsf		GPIO,Output				;Output = 1
	goto	loop
OutLo
	bcf		GPIO,Output				;Output = 0
	goto	loop
	
;Mode was 0 so convert to MPH. Start by counting rising edges on Tacho input
convert
	movf	GPIO,w					;read present pin states
	andlw	0x04					;isolate the tacho input signal
	xorwf	OldPinState,w			;compare with last check
	btfsc	STATUS,Z				;Z is clear if the input changed
	goto	loop
	movf	GPIO,w					;update OldPinState
	andlw	0x04
	movwf	OldPinState
	btfss	OldPinState,TachoIn		;only update the count if input is now high
	goto	loop
	incf	PulseCount,f			;add the new pulse to the count
	incf	CycleCount,f
	
;see which division ratio was selected. If Mode = 0 use 3/8, if = 1 use 64/103
	btfsc	GPIO,Precision				;skip if 3/8 mode
	goto	HiRes
	
;LoRes mode, supress pulses 0,3 and 5 in each 8 pulse sequence
;Start by counting pulses modulo 8
	movlw	0x08					;roll PulseCount to zero after 8 pulses
	subwf	PulseCount,w
	btfsc	STATUS,Z				;skip if the count wasn't 8
	clrf	PulseCount				;clear the count
	
;with only 3 values to check, it's quicker to just compare them
	movf	PulseCount,w
	btfsc	STATUS,Z				;skip if not zero
	goto	loop					;wait for next Tacho pulse
	movlw	3
	subwf	PulseCount,w
	btfsc	STATUS,Z				;skip if not 3
	goto	loop					;wait for next Tacho pulse
	movlw	5
	subwf	PulseCount,w
	btfsc	STATUS,Z				;skip if not 5
	goto	loop					;wait for next Tacho pulse
	goto	sendpulse				;send remaining pulses to the output


;HiRes mode, supress every third pulse and 6 others in every 103 pulse sequence
HiRes	
	movlw	.103					;roll PulseCount to zero after 103 pulses
	subwf	PulseCount,w
	btfsc	STATUS,Z				;skip if the count wasn't 103
	clrf	PulseCount				;clear the count
	movlw	.3
	subwf	CycleCount,w			;roll CycleCount to zero after 3 pulses
	btfsc	STATUS,Z
	clrf	CycleCount

	movlw	.24
	subwf	PulseCount,w			;always drop pulse 24
	btfsc	STATUS,Z
	goto	loop

	movlw	.38
	subwf	PulseCount,w			;always drop pulse 38
	btfsc	STATUS,Z
	goto	loop
	
	movlw	.52
	subwf	PulseCount,w			;always drop pulse 52
	btfsc	STATUS,Z
	goto	loop

	movlw	.64
	subwf	PulseCount,w			;always drop pulse 64
	btfsc	STATUS,Z
	goto	loop
	
	movlw	.88
	subwf	PulseCount,w			;always drop pulse 88
	btfsc	STATUS,Z
	goto	loop

	movlw	.94
	subwf	PulseCount,w			;always drop pulse 94
	btfsc	STATUS,Z
	goto	loop

	movlw	.102
	subwf	PulseCount,w			;always drop pulse 102
	btfsc	STATUS,Z;	
	goto	loop
	
	movf	CycleCount,f			;check if Cyclecount is zero
	btfsc	STATUS,Z				;skip if it isn't
	goto	loop
	goto	sendpulse
	
;send a pulse. make output high, wait for tacho input = 0 then make it low again.
sendpulse
	bsf		GPIO,Output				;make the output high
waitlow
	btfsc	GPIO,TachoIn			;copy tacho pulse to the output pin	
	goto	waitlow
	bcf		GPIO,Output				;make the output low again
	goto	loop
	
	dt		"(C)Brian Kelly 2010."
	END								;directive 'end of program'
